@@ -659,7 +659,7 @@ def gif_to_tensor(path, channels = 3, transform = T.ToTensor()):
     tensors = tuple(map(transform, seek_all_images(img, channels = channels)))
     return torch.stack(tensors, dim = 1)
 
-def identity(t):
+def identity(t, *args, **kwargs):
     return t
 
 def normalize_img(t):
@@ -668,6 +668,17 @@ def normalize_img(t):
 def unnormalize_img(t):
     return (t + 1) * 0.5
 
+def cast_num_frames(t, *, frames):
+    f = t.shape[1]
+
+    if f == frames:
+        return t
+
+    if f > frames:
+        return t[:, :frames]
+
+    return F.pad(t, (0, 0, 0, 0, 0, frames - f))
+
 class Dataset(data.Dataset):
     def __init__(
         self,
@@ -676,6 +687,7 @@ class Dataset(data.Dataset):
         channels = 3,
         num_frames = 16,
         horizontal_flip = False,
+        force_num_frames = True,
         exts = ['gif']
     ):
         super().__init__()
@@ -684,6 +696,8 @@ class Dataset(data.Dataset):
         self.channels = channels
         self.paths = [p for ext in exts for p in Path(f'{folder}').glob(f'**/*.{ext}')]
 
+        self.cast_num_frames_fn = partial(cast_num_frames, frames = num_frames) if force_num_frames else identity
+
         self.transform = T.Compose([
             T.Resize(image_size),
             T.RandomHorizontalFlip() if horizontal_flip else T.Lambda(identity),
@@ -697,10 +711,12 @@ class Dataset(data.Dataset):
 
     def __getitem__(self, index):
         path = self.paths[index]
-        return gif_to_tensor(path, self.channels, transform = self.transform)
+        tensor = gif_to_tensor(path, self.channels, transform = self.transform)
+        return self.cast_num_frames_fn(tensor)
 
 # trainer class
 
+import wandb
 class Trainer(object):
     def __init__(
         self,
@@ -776,6 +792,11 @@ class Trainer(object):
         torch.save(data, str(self.results_folder / f'model-{milestone}.pt'))
 
     def load(self, milestone):
+        if milestone == -1:
+            all_milestones = [int(p.stem.split('-')[-1]) for p in Path(self.results_folder).glob('**/*.pt')]
+            assert len(all_milestones) > 0, 'need to have at least one milestone to load from latest checkpoint (milestone == -1)'
+            milestone = max(all_milestones)
+
         data = torch.load(str(self.results_folder / f'model-{milestone}.pt'))
 
         self.step = data['step']
@@ -801,6 +822,7 @@ class Trainer(object):
 
                 print(f'{self.step}: {loss.item()}')
 
+            log = {'loss': loss.item()}
             self.scaler.step(self.opt)
             self.scaler.update()
             self.opt.zero_grad()
@@ -820,10 +842,12 @@ class Trainer(object):
                 all_videos_list = F.pad(all_videos_list, (2, 2, 2, 2))
 
                 one_gif = rearrange(all_videos_list, '(i j) c f h w -> c f (i h) (j w)', i = self.num_sample_rows)
-                video_tensor_to_gif(one_gif, str(self.results_folder / str(f'{milestone}.gif')))
-
+                video_path = str(self.results_folder / str(f'{milestone}.gif'))
+                video_tensor_to_gif(one_gif, video_path)
+                log = {**log, 'sample': wandb.Video(video_path)}
                 self.save(milestone)
 
+            wandb.log(log)
             self.step += 1
 
         print('training completed')
